<?php
require_once (__DIR__ . '/../php/op5/license/op5lic.php');
require_once ('op5/license/generator.php');

/**
 * Library tests for op5lic
 *
 * @package default
 *
 */
class op5_license_op5lic extends PHPUnit_Framework_TestCase {
	private $license_file;
	private $dbconn = false;
	const TRIAL_MESSAGE_REGEXP = '/Get access to tips, tutorials, support and more during your trial - click here/';
	const FREE_MESSAGE_REGEXP = '/Your free license is about to expire. You need to renew your license/';
	const UNLICENSED_MESSAGE_REGEXP = '/Your license has expired. You need to renew your license./';

	/**
	 * Constructor
	 *
	 * @return void
	 *
	 */
	function __construct($test_license_path = null) {
		if (!$test_license_path) {
			$this->license_file = "/tmp/op5_test_license.xml";
		} else {
			$this->license_file = $test_license_path;
		}
	}
	private function dbquery($query) {
		if ($this->dbconn === false) {
			$this->dbconn = new PDO('mysql:dbname=merlin;host=127.0.0.1',
				'merlin', 'merlin');
		}
		$this->dbconn->query($query);
	}

	/**
	 * Installs test-specific license files
	 *
	 * @param $license_data array
	 * @return void
	 *
	 */
	function mock($license_data, $version) {
		$licgen = new op5license_generator($version);
		$int = new op5license_generator_interval();

		if (isset($license_data['IssuedToCustomerNumber'])) {
			$licgen->set_user_info('customer.number',
				$license_data['IssuedToCustomerNumber']);
		} else {
			$licgen->set_user_info('customer.number', '999999-01');
		}
		unset($license_data['IssuedToCustomerNumber']);
		if (isset($license_data['IssuedToName'])) {
			$licgen->set_user_info('customer.name',
				$license_data['IssuedToName']);
		} else {
			$licgen->set_user_info('customer.name', 'TestCustomer');
		}
		unset($license_data['IssuedToName']);
		if (isset($license_data['IssuedToEmail'])) {
			$licgen->set_user_info('customer.email',
				$license_data['IssuedToEmail']);
		} else {
			$licgen->set_user_info('customer.email', 'test@test.com');
		}
		unset($license_data['IssuedToEmail']);

		if (!empty($license_data['IssueDate'])) {
			if (is_numeric($license_data['IssueDate'])) {
				$int->set_start_time(intval($license_data['IssueDate']));
			} else {
				$int->set_start_time(
					strtotime($license_data['IssueDate']) + 12 * 3600);
			}
		}
		unset($license_data['IssueDate']);
		if (!empty($license_data['ExpireDate'])) {
			if (is_numeric($license_data['ExpireDate'])) {
				$int->set_end_time(intval($license_data['ExpireDate']));
			} else {
				$int->set_end_time(
					strtotime($license_data['ExpireDate']) + 12 * 3600);
			}
		}
		unset($license_data['ExpireDate']);

		foreach ($license_data as $k => $v) {
			$int->set_metric($k, $v);
		}

		$licgen->add_interval($int);
		$licgen->generate();

		$file = file_put_contents($this->license_file, $licgen->get_license());
		if ($file === false || $file < 1) {
			throw new Exception(
				'An error occured while trying to create license file', 1);
		}
	}

	/**
	 * Seeds database with report_data
	 */
	function seedDataBase($age = 0) {
		$this->clearDataBase();

		$starttime = time() - $age;
		$this->dbquery(
			"INSERT INTO report_data (timestamp, event_type, host_name, service_description, state, hard, retry) VALUES (" .
				 $starttime . ", 100, '', '', 0, 0, 0)");
	}

	/**
	 * Removes database entry
	 */
	function clearDataBase() {
		$this->dbquery("TRUNCATE TABLE report_data");
	}

	/**
	 * Modify license values
	 * Invalidates license signature
	 *
	 * @return void
	 *
	 */
	function modify_license($field) {
		$xml = file_get_contents($this->license_file);
		$xml = preg_replace('/Regular/', 'Enterprise', $xml);
		file_put_contents($this->license_file, $xml);
	}

	/**
	 * Removal of mocked data after every test
	 */
	function tearDown() {
		if (is_file($this->license_file)) {
			unlink($this->license_file);
		}
		// $this->clearDataBaseEntry();
	}

	/**
	 * A valid license file shouldn't return any warnings
	 * Tests signature verification
	 */
	function test_valid_op5v1_license() {
		$this->mock(array (), 'op5v1');
		$lic = new op5lic($this->license_file);
		$customer = $lic->fetch_customer_info();
		$this->assertSame(false,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(false, $lic->fetch_property('unlicensed'));
		$this->assertSame(false, $lic->fetch_property('unlimited'));
		$this->assertSame('999999-01', $customer['number']);
		$this->assertSame('TestCustomer', $customer['name']);
		$this->assertSame('test@test.com', $customer['email']);
	}

	/**
	 * A valid license file shouldn't return any warnings
	 * Tests signature verification
	 */
	function test_valid_op5v2_license() {
		$this->mock(array (), 'op5v2');
		$lic = new op5lic($this->license_file);
		$customer = $lic->fetch_customer_info();
		$this->assertSame(false,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(false, $lic->fetch_property('unlicensed'));
		$this->assertSame(false, $lic->fetch_property('unlimited'));
		$this->assertSame('999999-01', $customer['number']);
		$this->assertSame('TestCustomer', $customer['name']);
	}

	/**
	 * A valid license file shouldn't return any warnings
	 * Tests signature verification
	 */
	function test_invalid_op5v1_license() {
		$this->mock(array (), 'op5v1');

		/* Trash the signature */
		$licfile = file_get_contents($this->license_file);
		$signature = base64_encode('Invalidsignature');
		$licfile = preg_replace(
			':<signature value="(.*)" key="op5-2007a"></signature>:',
			'<signature value="' . $signature . '" key="op5-2007a"></signature>',
			$licfile);
		file_put_contents($this->license_file, $licfile);

		$lic = new op5lic($this->license_file);
		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(true, $lic->fetch_property('unlicensed'));
		$this->assertSame(false, $lic->fetch_property('unlimited'));
		$this->assertEquals(array ("Invalid license signature"),
			$lic->fetch_load_errors());
	}

	/**
	 * A valid license file shouldn't return any warnings
	 * Tests signature verification
	 */
	function test_invalid_op5v2_license() {
		$this->mock(array (), 'op5v2');

		/* Trash the signature */
		$licfile = file_get_contents($this->license_file);
		$signature = base64_encode('Invalidsignature');
		$licfile = preg_replace(
			':<signature value="(.*)" key="op5-2007a"></signature>:',
			'<signature value="' . $signature . '" key="op5-2007a"></signature>',
			$licfile);
		file_put_contents($this->license_file, $licfile);

		$lic = new op5lic($this->license_file);
		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(true, $lic->fetch_property('unlicensed'));
		$this->assertSame(false, $lic->fetch_property('unlimited'));
		$this->assertEquals(array ("Invalid license signature"),
			$lic->fetch_load_errors());
	}

	/**
	 * Verify that metrics is read correctly for op5v1
	 */
	function test_valid_op5v1_metrics() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'trapper' => 1,'monitor.pollers' => 7),
			'op5v1');
		$lic = new op5lic($this->license_file);
		$this->assertSame(37, $lic->fetch_metric('monitor'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor.service'));
		$this->assertSame(0, $lic->fetch_metric('logserver'));
		$this->assertSame(1, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(7, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that an op5v1 license without extension implicitly provides some
	 * extensions
	 */
	function test_valid_op5v1_wo_extensions() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(array ('monitor.hosts' => 37,'meta.noextension' => 1),
			'op5v1');
		$lic = new op5lic($this->license_file);
		$this->assertSame(37, $lic->fetch_metric('monitor'));
		$this->assertGreaterThanOrEqual(1, $lic->fetch_metric('aps'));
		$this->assertGreaterThanOrEqual(1, $lic->fetch_metric('peers'));
		$this->assertGreaterThanOrEqual(1, $lic->fetch_metric('pollers'));
		$this->assertGreaterThanOrEqual(1, $lic->fetch_metric('trapper'));
	}

	/**
	 * Verify that messages is empty for valid op5v2
	 */
	function test_valid_op5v2_messages() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'trapper' => true,
				'monitor.pollers' => 7), 'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertSame(array (), $lic->fetch_messages('load'));
		$this->assertSame(array (), $lic->fetch_messages('report'));
	}

	/**
	 * Verify that messages is empty for valid op5v1
	 */
	function test_valid_op5v1_messages() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'trapper' => true,
				'monitor.pollers' => 7), 'op5v1');
		$lic = new op5lic($this->license_file);
		$this->assertSame(array (), $lic->fetch_messages('load'));
		$this->assertSame(array (), $lic->fetch_messages('report'));
	}

	/**
	 * Verify that metrics is read correctly for op5v2
	 */
	function test_valid_op5v2_metrics() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'trapper' => 1,'monitor.pollers' => 7),
			'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertSame(37, $lic->fetch_metric('monitor'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor.service'));
		$this->assertSame(0, $lic->fetch_metric('logserver'));
		$this->assertSame(1, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(7, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that metrics is read correctly for op5v1
	 */
	function test_valid_op5v1_metrics_yes_no() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'trapper' => true,
				'monitor.pollers' => false), 'op5v1');
		$lic = new op5lic($this->license_file);
		$this->assertSame(37, $lic->fetch_metric('monitor'));
		$this->assertSame(0, $lic->fetch_metric('logserver'));
		$this->assertSame(1, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(0, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that metrics is read correctly for op5v2
	 */
	function test_valid_op5v2_metrics_yes_no() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'trapper' => 'yes',
				'monitor.pollers' => 'no'), 'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertSame(37, $lic->fetch_metric('monitor'));
		$this->assertSame(0, $lic->fetch_metric('logserver'));
		$this->assertSame(1, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(0, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that metrics is read correctly for op5v1
	 */
	function test_valid_op5v1_logserver_metrics() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'logserver.hosts' => 12,
				'trapper' => true,'monitor.pollers' => 7), 'op5v1');
		$lic = new op5lic($this->license_file);
		$this->assertSame(37, $lic->fetch_metric('monitor'));
		$this->assertSame(12, $lic->fetch_metric('logserver'));
		$this->assertSame(1, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(7, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that metrics is read correctly for op5v2
	 */
	function test_valid_op5v2_logserver_metrics() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'logserver.hosts' => 12,
				'logserver' => true,'trapper' => true,'monitor.pollers' => 7),
			'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertSame(37, $lic->fetch_metric('monitor'));
		$this->assertSame(12, $lic->fetch_metric('logserver'));
		$this->assertSame(1, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(7, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that metrics is read correctly for op5v2 site licenses
	 *
	 * only the specific license type metric should go to infinity.
	 */
	function test_valid_op5v2_site_metrics() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'logserver.hosts' => 12,
				'logserver' => true,'monitor.type' => 'Site','trapper' => true,
				'monitor.pollers' => 7), 'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor.service'));
		$this->assertSame(12, $lic->fetch_metric('logserver'));
		$this->assertSame(1, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(7, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that metrics is read correctly for op5v2 with unknown type
	 *
	 * Unknown license types invalidates the license, and treats it as
	 * unlicensed
	 */
	function test_valid_op5v2_unknown_type() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(array ('monitor.type' => 'UnknownLicenseType'), 'op5v2');
		$lic = new op5lic($this->license_file);

		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(true, $lic->fetch_property('unlicensed'));
		$this->assertSame(false, $lic->fetch_property('unlimited'));
	}

	/**
	 * Verify that metrics is read correctly for op5v1 site license
	 *
	 * only the specific license type metric should go to infinity.
	 */
	function test_valid_op5v1_site_metrics() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'logserver.hosts' => 12,
				'monitor.type' => 'Site','trapper' => true,
				'monitor.pollers' => 7), 'op5v1');
		$lic = new op5lic($this->license_file);

		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor.service'));
		$this->assertSame(12, $lic->fetch_metric('logserver'));
		$this->assertSame(1, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(7, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that fallback on unlicensed works for unknown license type
	 */
	function test_valid_op5v1_unknown_license_type() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.type' => 'InvalidLicenseType',
				'LogserverLicenseType' => 'InvalidLicenseType'), 'op5v1');
		$lic = new op5lic($this->license_file);
		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(true, $lic->fetch_property('unlicensed'));
		$this->assertSame(false, $lic->fetch_property('unlimited'));
	}

	/**
	 */
	function test_invalid_verifier() {
		$this->mock(array (), 'op5v2'); // Mock a op5v2 license
		                                // The verifier string shouldn't be a part
		                                // of the hash that
		                                // is signed
		$license = file_get_contents($this->license_file);
		$license = str_replace('verifier="op5v2"', 'verifier="invalid"',
			$license);
		file_put_contents($this->license_file, $license);
		$lic = new op5lic($this->license_file);
		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(true, $lic->fetch_property('unlicensed'));
		$this->assertSame(false, $lic->fetch_property('unlimited'));
		$this->assertEquals(array ("Error reading license content"),
			$lic->fetch_load_errors());
	}

	/**
	 * Verify that metrics is read correctly for op5v1
	 */
	function test_valid_trial_metrics() {
		$this->seedDataBase(29 * 24 * 3600);
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(false, $lic->fetch_property('unlicensed'));
		$this->assertSame(true, $lic->fetch_property('unlimited'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor.service'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('logserver'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('trapper'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor.peers'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor.pollers'));
	}

	/**
	 * Verify that report messages is read correctly for trial license, during
	 * entire period
	 */
	function test_valid_trial_report_messages() {
		for ($i = 0; $i < 30; $i++) {
			$this->seedDataBase($i * 24 * 3600);
			$lic = new op5lic('/this/file/doesnt/exist');
			$this->assertSame(
				array (
					'This report was generated using a trial license of op5 Monitor.'),
				$lic->fetch_messages('report'));
		}
	}

	/**
	 * Verify that metrics is read correctly for op5v1
	 */
	function test_valid_trial_empty_db_metrics() {
		$this->clearDataBase();
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(false, $lic->fetch_property('unlicensed'));
		$this->assertSame(true, $lic->fetch_property('unlimited'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('monitor'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('logserver'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('trapper'));
		$this->assertSame(op5lic::METRIC_INFINITY, $lic->fetch_metric('peers'));
		$this->assertSame(op5lic::METRIC_INFINITY,
			$lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that expired trial license gives correct metrics
	 */
	function test_expired_trial_metrics() {
		$this->seedDataBase(31 * 24 * 3600);
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'));
		$this->assertSame(true, $lic->fetch_property('unlicensed'));
		$this->assertSame(false, $lic->fetch_property('unlimited'));
		$this->assertSame(0, $lic->fetch_metric('monitor'));
		$this->assertSame(0, $lic->fetch_metric('logserver'));
		$this->assertSame(0, $lic->fetch_metric('trapper'));
		$this->assertSame(0, $lic->fetch_metric('peers'));
		$this->assertSame(0, $lic->fetch_metric('pollers'));
	}

	/**
	 * Verify that expired trial license gives correct messgaes
	 */
	function test_expired_trial_messages() {
		$this->seedDataBase(31 * 24 * 3600);
		$lic = new op5lic('/this/file/doesnt/exist');
		$msg = $lic->fetch_load_messages();
		$this->assertSame(1, count($msg));
		$this->assertRegexp(self::UNLICENSED_MESSAGE_REGEXP, $msg[0]);
		$this->assertSame(array (), $lic->fetch_messages('report'));
	}

	/**
	 * Verify that metrics is read correctly for op5v2
	 */
	function test_license_metrics_matches_fetch_metric() {
		/*
		 * Note: This doesn't test all metrics, but should test all ways metrics
		 * is fetched
		 */
		$this->mock(
			array ('monitor.hosts' => 37,'logserver.hosts' => 12,
				'logserver' => true,'trapper' => true,'monitor.pollers' => 7),
			'op5v2');
		$lic = new op5lic($this->license_file);
		$metrics = $lic->fetch_license_metrics();
		$this->assertSame($lic->fetch_metric('monitor'), $metrics['monitor']);
		$this->assertSame($lic->fetch_metric('logserver'),
			$metrics['logserver']);
		$this->assertSame($lic->fetch_metric('trapper'), $metrics['trapper']);
		$this->assertSame($lic->fetch_metric('peers'), $metrics['peers']);
		$this->assertSame($lic->fetch_metric('pollers'), $metrics['pollers']);
	}

	/**
	 * Verify that metrics is read correctly for op5v2 valid free
	 */
	function test_valid_free_metrics() {
		$this->seedDataBase(0);
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now + 17 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Free'), 'op5v2');

		$lic = new op5lic($this->license_file);

		$this->assertThat($time_issue,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_issue')),
				$this->lessThan($lic->fetch_metric('time_issue') + 60 * 60 * 24)),
			'time_issue');

		/* expire tomorrow, make expiration day valid */
		$this->assertThat($time_expire + 24 * 60 * 60,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_expire')),
				$this->lessThan(
					$lic->fetch_metric('time_expire') + 60 * 60 * 24)),
			'time_expire');

		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'),
			'enable_extra_license_info');
		$this->assertSame(false, $lic->fetch_property('unlicensed'),
			'unlicensed');
		$this->assertSame(false, $lic->fetch_property('unlimited'), 'unlimited');
		$this->assertSame(100, $lic->fetch_metric('monitor'), 'monitor');
		$this->assertSame(0, $lic->fetch_metric('logserver'), 'logserver');
		$this->assertSame(0, $lic->fetch_metric('trapper'), 'trapper');
		$this->assertSame(0, $lic->fetch_metric('peers'), 'monitor.peers');
		$this->assertSame(0, $lic->fetch_metric('pollers'), 'monitor.pollers');
	}

	/**
	 * Verify that report messages is read correctly for trial license, during
	 * entire period
	 */
	function test_valid_free_report_messages() {
		$this->seedDataBase(0);
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now + 17 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Free'), 'op5v2');

		$lic = new op5lic($this->license_file);
		$this->assertSame(
			array (
				'This report was generated using a free license of op5 Monitor.'),
			$lic->fetch_messages('report'));
	}

	/**
	 * Verify that op5v2 expired free license is unlicensed
	 */
	function test_expired_free_unlicensed() {
		$this->seedDataBase(0);
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now - 17 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Free'), 'op5v2');

		$lic = new op5lic($this->license_file);

		$this->assertThat($time_issue,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_issue')),
				$this->lessThan($lic->fetch_metric('time_issue') + 60 * 60 * 24)),
			'time_issue');

		/* expire tomorrow, make expiration day valid */
		$this->assertThat($time_expire + 24 * 60 * 60,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_expire')),
				$this->lessThan(
					$lic->fetch_metric('time_expire') + 60 * 60 * 24)),
			'time_expire');

		$this->assertSame(true,
			$lic->fetch_property('enable_extra_license_info'),
			'enable_extra_license_info');
		$this->assertSame(true, $lic->fetch_property('unlicensed'),
			'unlicensed');
		$this->assertSame(false, $lic->fetch_property('unlimited'), 'unlimited');
		$this->assertSame(0, $lic->fetch_metric('monitor'), 'monitor');
		$this->assertSame(0, $lic->fetch_metric('logserver'), 'logserver');
		$this->assertSame(0, $lic->fetch_metric('trapper'), 'trapper');
		$this->assertSame(0, $lic->fetch_metric('peers'), 'peers');
		$this->assertSame(0, $lic->fetch_metric('pollers'), 'pollers');
	}

	/**
	 * Verify that an expired free license gives correct messages
	 */
	function test_expired_free_messages() {
		$this->seedDataBase(0);
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now - 17 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Free'), 'op5v2');

		$lic = new op5lic($this->license_file);
		$msg = $lic->fetch_load_messages();
		$this->assertSame(1, count($msg));
		$this->assertRegexp(self::UNLICENSED_MESSAGE_REGEXP, $msg[0]);
		$this->assertSame(
			array (
				'This report was generated using a free license of op5 Monitor.'),
			$lic->fetch_messages('report'));
	}

	/**
	 * Test that a regular license with long time left doesn't output a nagbar
	 */
	function test_simple_query_regular_long_time_left() {
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now + 17 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Regular'),
			'op5v2');

		$lic = new op5lic($this->license_file);
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(0, count($msg));
	}

	/**
	 * Test that a regular license close to expiration displays a nagbar
	 */
	function test_simple_query_regular_short_time_left() {
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now + 13 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Regular'),
			'op5v2');

		$lic = new op5lic($this->license_file);
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(0, count($msg));
	}

	/**
	 * Test that a free license with long time left doesn't output a nagbar
	 */
	function test_simple_query_free_long_time_left() {
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now + 17 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Free'), 'op5v2');

		$lic = new op5lic($this->license_file);
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(0, count($msg));
	}

	/**
	 * Test that a free license close to expiration displays a nagbar
	 */
	function test_simple_query_free_short_time_left() {
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now + 13 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Free'), 'op5v2');

		$lic = new op5lic($this->license_file);
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(2, count($msg));
		$this->assertRegexp(self::FREE_MESSAGE_REGEXP, $msg[0]);
		$this->assertRegexp('/^Your license will expire in/', $msg[1]);
	}

	/**
	 * Test that a trial license close to expiration displays only one unit of
	 * precision in its second message
	 */
	function test_time_left_precision() {
		/* Make it irrational (this is about 1d17h left) */
		$this->seedDataBase(M_PI * 9 * 24 * 3600);
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(2, count($msg));
		$this->assertRegexp(self::TRIAL_MESSAGE_REGEXP, $msg[0]);
		/* Only one int in this message */
		$this->assertRegexp('/^[^0-9]*expire[^0-9]*[0-9]+[^0-9]*$/', $msg[1]);
	}

	/**
	 * Verify that expiration date is entire day, inclusive, in op5v2
	 */
	function test_expiration_day_inclusive() {
		$this->seedDataBase(0);
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Free'), 'op5v2');

		$lic = new op5lic($this->license_file);

		$this->assertThat($time_issue,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_issue')),
				$this->lessThan($lic->fetch_metric('time_issue') + 60 * 60 * 24)),
			'time_issue');

		/* expire tomorrow, make expiration day valid */
		$this->assertThat($time_expire + 24 * 60 * 60,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_expire')),
				$this->lessThan(
					$lic->fetch_metric('time_expire') + 60 * 60 * 24)),
			'time_expire');

		/* It shouldn't be unlicensed if expires exactly now... */
		$this->assertSame(false, $lic->fetch_property('unlicensed'),
			'unlicensed');
	}

	/**
	 * Verify that metrics is read correctly for op5v2 valid free
	 */
	function test_valid_op5v2_timelimited_metrics() {
		$this->seedDataBase(0);
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now + 17 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Regular'),
			'op5v2');

		$lic = new op5lic($this->license_file);

		$this->assertThat($time_issue,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_issue')),
				$this->lessThan($lic->fetch_metric('time_issue') + 60 * 60 * 24)),
			'time_issue');

		/* expire tomorrow, make expiration day valid */
		$this->assertThat($time_expire + 24 * 60 * 60,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_expire')),
				$this->lessThan(
					$lic->fetch_metric('time_expire') + 60 * 60 * 24)),
			'time_expire');

		$this->assertSame(false,
			$lic->fetch_property('enable_extra_license_info'),
			'enable_extra_license_info');
		$this->assertSame(false, $lic->fetch_property('unlicensed'),
			'unlicensed');
		$this->assertSame(false, $lic->fetch_property('unlimited'), 'unlimited');
		$this->assertSame(100, $lic->fetch_metric('monitor'), 'monitor');
		$this->assertSame(0, $lic->fetch_metric('logserver'), 'logserver');
		$this->assertSame(0, $lic->fetch_metric('trapper'), 'trapper');
		$this->assertSame(0, $lic->fetch_metric('peers'), 'peers');
		$this->assertSame(0, $lic->fetch_metric('pollers'), 'pollers');
	}

	/**
	 * Verify that op5v2 expired free license is unlicensed
	 */
	function test_expired_op5v2_timelimited_metrics() {
		$this->seedDataBase(0);
		$now = time();
		$time_issue = $now - 100 * 24 * 60 * 60;
		$time_expire = $now - 17 * 24 * 60 * 60;
		$this->mock(
			array ('monitor.hosts' => 100,'IssueDate' => $time_issue,
				'ExpireDate' => $time_expire,'monitor.type' => 'Regular'),
			'op5v2');

		$lic = new op5lic($this->license_file);

		$this->assertThat($time_issue,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_issue')),
				$this->lessThan($lic->fetch_metric('time_issue') + 60 * 60 * 24)),
			'time_issue');

		/* expire tomorrow, make expiration day valid */
		$this->assertThat($time_expire + 24 * 60 * 60,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_expire')),
				$this->lessThan(
					$lic->fetch_metric('time_expire') + 60 * 60 * 24)),
			'time_expire');

		$this->assertTrue(
			$lic->query(array ('monitor' => 100,'time_expire' => false), $msg));
		$this->assertSame(1, count($msg));
		$this->assertRegexp(
			'/Your license has expired. You need to renew your license./',
			$msg[0]);

		$this->assertSame(false,
			$lic->fetch_property('enable_extra_license_info'),
			'enable_extra_license_info');
		$this->assertSame(false, $lic->fetch_property('unlicensed'),
			'unlicensed');
		$this->assertSame(false, $lic->fetch_property('unlimited'), 'unlimited');

		$this->assertSame(100, $lic->fetch_metric('monitor'), 'monitor');
		$this->assertSame(0, $lic->fetch_metric('logserver'), 'logserver');
		$this->assertSame(0, $lic->fetch_metric('trapper'), 'trapper');
		$this->assertSame(0, $lic->fetch_metric('peers'), 'peers');
		$this->assertSame(0, $lic->fetch_metric('pollers'), 'pollers');

		$this->assertFalse($lic->query(array ('monitor' => 200)));
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_simple_query() {
		$this->mock(array ('monitor.hosts' => 37,'logserver.hosts' => 2),
			'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertTrue(
			$lic->query(array ('Monitor' => 37,'time_expire' => false), $msg));
		$this->assertSame(array (), $msg);
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_simple_query_exceeded() {
		$this->mock(array ('monitor.hosts' => 37,'logserver.hosts' => 2),
			'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertFalse(
			$lic->query(array ('Monitor' => 38,'time_expire' => false), $msg));
		$this->assertSame(
			array ("Your usage for monitor is 38, your license allows 37"), $msg);
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_query_multi_attr() {
		$this->mock(
			array ('monitor.hosts' => 37,'logserver.hosts' => 2,
				'logserver' => true), 'op5v2');
		$lic = new op5lic($this->license_file);

		$this->assertTrue(
			$lic->query(
				array ('Monitor' => 37,'LogServer' => 2,'time_expire' => false),
				$msg));
		$this->assertSame(array (), $msg);

		$this->assertFalse(
			$lic->query(
				array ('Monitor' => 38,'LogServer' => 2,'time_expire' => false),
				$msg));
		$this->assertSame(
			array ('Your usage for monitor is 38, your license allows 37'), $msg);

		$this->assertFalse(
			$lic->query(
				array ('Monitor' => 37,'LogServer' => 3,'time_expire' => false),
				$msg));
		$this->assertSame(
			array ('Your usage for logserver is 3, your license allows 2'), $msg);

		$this->assertFalse(
			$lic->query(
				array ('Monitor' => 38,'LogServer' => 3,'time_expire' => false),
				$msg));
		$this->assertSame(
			array ("Your usage for monitor is 38, your license allows 37",
				"Your usage for logserver is 3, your license allows 2"), $msg);
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_simple_query_trial() {
		$this->seedDataBase(29 * 24 * 3600);
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(2, count($msg));
		$this->assertRegexp(self::TRIAL_MESSAGE_REGEXP, $msg[0]);
		$this->assertRegexp('/^Your license will expire in/', $msg[1]);
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_trial_no_mon() {
		/* Make sure mon command returns an expired trial (see other test) */
		$this->seedDataBase(31 * 24 * 3600);

		/* Mask mon command, to test fallback in trial start time */
		$pathenv = getenv('PATH');
		putenv('PATH=');

		$lic = new op5lic('/this/file/doesnt/exist');

		/* Reset environment */
		putenv('PATH=' . $pathenv);

		/* Many hosts, and no message; trial time has just started */
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(0, count($msg));
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_trial_notice_bar_appearance() {
		$this->seedDataBase(3 * 24 * 3600 + 300);
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(2, count($msg));
		$this->assertRegexp(self::TRIAL_MESSAGE_REGEXP, $msg[0]);
		$this->assertRegexp('/^Your license will expire in/', $msg[1]);

		$this->seedDataBase(3 * 24 * 3600 - 300);
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(0, count($msg));
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_trial_time_expire_metric() {

		/* Some test assumptions to mock */
		$trial_time_length = 30*24*60*60;
		$time_since_install = 13*24*3600;
		$time_error_margin = 5 * 60;

		/* Mock up license */
		$time_expire = time() - $time_since_install + $trial_time_length;

		$this->seedDataBase($time_since_install);
		$lic = new op5lic('/this/file/doesnt/exist');

		/* Run the test case */
		$this->assertThat($time_expire,
			$this->logicalAnd(
				$this->greaterThan($lic->fetch_metric('time_expire') - $time_error_margin),
				$this->lessThan($lic->fetch_metric('time_expire') + $time_error_margin)),
			'time_expire');
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_simple_query_trial_expired() {
		$this->seedDataBase(31 * 24 * 3600);
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertFalse($lic->query(array ('Monitor' => 77), $msg));
		$this->assertTrue($lic->fetch_property('unlicensed'));
		$this->assertSame(2, count($msg));
		/* Those messages is visible only in the portal page */
		/* This implicitly verifies that metrics isn't visible if unlicensed */
		$this->assertRegexp(self::UNLICENSED_MESSAGE_REGEXP, $msg[0]);
		$this->assertRegexp('/^Your license expired .* ago$/', $msg[1]);
	}

	/**
	 * Test that a simple valid query is ok
	 */
	function test_simple_query_trial_start_undefined() {
		$this->clearDataBase();
		$lic = new op5lic('/this/file/doesnt/exist');
		$this->assertTrue($lic->query(array ('Monitor' => 77), $msg));
		$this->assertSame(0, count($msg));
	}

	/**
	 * Test that query_threshold works for finite metric
	 */
	function test_query_finite() {
		$this->mock(array ('monitor.hosts' => 100), 'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertSame(true, $lic->query_threshold('monitor', 0, 0.7, $msg));
		$this->assertSame(
			array ("You are using 0.0% of your license for monitor"), $msg);
		$this->assertSame(true, $lic->query_threshold('monitor', 69, 0.7, $msg));
		$this->assertSame(
			array ("You are using 69.0% of your license for monitor"), $msg);
		$this->assertSame(false,
			$lic->query_threshold('monitor', 70, 0.7, $msg));
		$this->assertSame(
			array (
				"You are using 70.0% (70 out of 100) of your license for monitor"),
			$msg);
		$this->assertSame(false,
			$lic->query_threshold('monitor', 71, 0.7, $msg));
		$this->assertSame(
			array (
				"You are using 71.0% (71 out of 100) of your license for monitor"),
			$msg);
		$this->assertSame(false,
			$lic->query_threshold('monitor', 100, 0.7, $msg));
		$this->assertSame(
			array (
				"You are using 100.0% (100 out of 100) of your license for monitor"),
			$msg);
		$this->assertSame(false,
			$lic->query_threshold('monitor', 101, 0.7, $msg));
		$this->assertSame(
			array (
				"You are exceeding your license (you use 101, you may use 100) for monitor"),
			$msg);
		$this->assertSame(false,
			$lic->query_threshold('badboll of doom', 1, 0.7, $msg));
		$this->assertSame(
			array (
				"You are not licensed to have any badboll of doom, you use 1"),
			$msg);
		$this->assertSame(true,
			$lic->query_threshold('badboll of doom', 0, 0.7, $msg));
		$this->assertSame(
			array ("You are using 0.0% of your license for badboll of doom"),
			$msg);
	}

	/**
	 * Test that query_threshold works for finite metric
	 */
	function test_query_infinite() {
		$this->mock(array ('monitor.hosts' => 1,'monitor.type' => 'Site'),
			'op5v1');
		$lic = new op5lic($this->license_file);
		$this->assertSame(true, $lic->query_threshold('monitor', 0, 0.7, $msg));
		$this->assertSame(array ("No limitation in license size for monitor"),
			$msg);
		$this->assertSame(true,
			$lic->query_threshold('monitor', 100, 0.7, $msg));
		$this->assertSame(array ("No limitation in license size for monitor"),
			$msg);
	}

	/**
	 * A valid license file shouldn't return any warnings
	 * Tests signature verification
	 */
	function test_undefined_properties() {
		$this->mock(array (), 'op5v2');
		$lic = new op5lic($this->license_file);
		$this->assertSame(false, $lic->fetch_property('this_is_not_valid'));
	}
} // END class LibraryTest
